@@ -345,6 +345,11 @@ void log_errno(log_msg_priority_t priority, const char *fmt, ...)
 
 int text_handler_init(void)
 {
+#ifdef HAVE_SETVBUF
+  /* Set stdout to unbuffered mode */
+  setvbuf(stdout, NULL, _IONBF, 0);
+#endif
+  
   verbosity = sb_get_value_int("verbosity");
 
   if (verbosity > LOG_DEBUG)
@@ -409,7 +414,7 @@ int text_handler_process(log_msg_t *msg)
   }
   
   printf("%s%s", prefix, text_msg->text);
-
+  
   return 0;
 }
 
@@ -441,15 +446,17 @@ int oper_handler_init(void)
 
   if (batch_mode)
   {
+    int err;
     pthread_mutex_init(&batch_mutex, NULL);
     pthread_cond_init(&batch_cond, NULL);
 
     /* Create batch thread */
     pthread_attr_init(&batch_attr);
-    if (pthread_create(&batch_thread, &batch_attr, &batch_runner_proc, NULL)
+    if ((err = pthread_create(&batch_thread, &batch_attr, &batch_runner_proc, NULL))
         != 0)
     {
-      log_errno(LOG_FATAL, "Batch thread creation failed");
+      log_text(LOG_FATAL, "Batch thread creation failed, errno = %d (%s)",
+                err, strerror(err));
       return 1;
     }
     batch_status = BATCH_STATUS_STOP;
@@ -541,15 +548,17 @@ int oper_handler_done(void)
 
   if (batch_mode)
   {
+    int err;
     /* Stop the batch thread */
     pthread_mutex_lock(&batch_mutex);
     batch_status = BATCH_STATUS_STOP;
     pthread_cond_signal(&batch_cond);
     pthread_mutex_unlock(&batch_mutex);
 
-    if (pthread_join(batch_thread, NULL))
+    if ((err = pthread_join(batch_thread, NULL)))
     {
-      log_errno(LOG_FATAL, "Batch thread join failed");
+      log_text(LOG_FATAL, "Batch thread join failed, errno = %d (%s)",
+               err, strerror(err));
       return 1;
     }
 
@@ -573,12 +582,12 @@ int oper_handler_done(void)
            NS2SEC(get_sum_time(&t)));
 
   log_text(LOG_NOTICE, "    per-request statistics:");
-  log_text(LOG_NOTICE, "         min:                            %.4fs",
-           NS2SEC(get_min_time(&t)));
-  log_text(LOG_NOTICE, "         avg:                            %.4fs",
-           NS2SEC(get_avg_time(&t)));
-  log_text(LOG_NOTICE, "         max:                            %.4fs",
-           NS2SEC(get_max_time(&t)));
+  log_text(LOG_NOTICE, "         min:                            %10.2fms",
+           NS2MS(get_min_time(&t)));
+  log_text(LOG_NOTICE, "         avg:                            %10.2fms",
+           NS2MS(get_avg_time(&t)));
+  log_text(LOG_NOTICE, "         max:                            %10.2fms",
+           NS2MS(get_max_time(&t)));
 
   /* Print approx. percentile value for event execution times */
   if (t.events > 0)
@@ -599,8 +608,8 @@ int oper_handler_done(void)
   
     /* Calculate response time corresponding to this element */
     optime = exp((double)i / oper_log_mult + oper_log_deduct);
-    log_text(LOG_NOTICE, "         approx. %3d percentile:         %.4fs",
-             oper_percentile, NS2SEC(optime));
+    log_text(LOG_NOTICE, "         approx. %3d percentile:         %10.2fms",
+             oper_percentile, NS2MS(optime));
   }
   log_text(LOG_NOTICE, "");
 
